//------------------------------------------------------------------
//
//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 1
#property indicator_color1  PaleVioletRed
#property indicator_width1  2
#property indicator_minimum -5
#property indicator_maximum 105

//
//
//
//
//

extern string TimeFrame         = "Current time frame";
extern int    RwmaDepth         = 10;
extern int    RwmaLength        = 2;
extern int    RwmaPrice         = PRICE_CLOSE;
extern int    StochasticLength  = 30;
extern int    StochasticSlowing =  5;
extern double LevelUp           = 80;
extern double LevelDown         = 20;
extern bool   alertsOn          = False;
extern bool   alertsOnCurrent   = true;
extern bool   alertsMessage     = true;
extern bool   alertsSound       = false;
extern bool   alertsEmail       = false;
extern bool   arrowsOn          = false;
extern string arrowsIdentifier  = "ifts arrows";
extern color  arrowsUpColor     = LimeGreen;
extern color  arrowsDnColor     = OrangeRed;
extern double arrowsUpGap       = 0.75;
extern double arrowsDownGap     = 0.75;
extern bool   Interpolate       = true;

//
//
//
//
//

#define MAX_depth 50
double  ma[];
double  stoch1[];
double  stoch2[];
double  ift[];
double  trend[];

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,ift);
   SetIndexBuffer(1,stoch1);
   SetIndexBuffer(2,stoch2);
   SetIndexBuffer(3,ma);
   SetIndexBuffer(4,trend);
      RwmaLength = MathMax(RwmaLength,2);
      RwmaDepth  = MathMax(MathMin(RwmaDepth,MAX_depth),2);
   
      //
      //
      //
      //
      //
      
         indicatorFileName = WindowExpertName();
         calculateValue    = TimeFrame=="calculateValue"; if (calculateValue) { return(0); }
         returnBars        = TimeFrame=="returnBars";     if (returnBars)     { return(0); }
         timeFrame         = stringToTimeFrame(TimeFrame);
         SetLevelValue(0,LevelUp);
         SetLevelValue(1,LevelDown);
      
      //
      //
      //
      //
      //

   IndicatorShortName(timeFrameToString(timeFrame)+" IFT stochastic of RWMA ("+StochasticLength+","+StochasticSlowing+","+RwmaLength+","+RwmaDepth+")");
   return(0);
}

//
//
//
//
//

int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

double pBuffer[][MAX_depth];
int start()
{
   double weight = (RwmaLength+1)*RwmaLength*0.5;
   int i,j,n,r,counted_bars=IndicatorCounted();

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { ift[0] = MathMin(limit+1,Bars-1); return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame == Period())
   {
      if (ArrayRange(pBuffer,0) != Bars) ArrayResize(pBuffer,Bars);
      for(i=limit, r=Bars-limit-1; i>=0; i--,r++)
      {
         double price = iMA(NULL,0,1,0,MODE_SMA,RwmaPrice,i);
         double coeff = 5;
         double sum   = 0;
         double sumc  = 0;
         for (int k=0; k<RwmaDepth; k++)
         {
            pBuffer[r][k] = price;
            if (r>RwmaLength)
            {
               for (j=0,price=0; j<RwmaLength; j++) price += (RwmaLength-j)*pBuffer[r-j][k];
                                                    price /= weight;
            }
            sum  += coeff*price;
            sumc += coeff;
            coeff = MathMax(coeff-1,1);
         }
         ma[i] = sum/sumc;
      
         //
         //
         //
         //
         //
      
         double min = ma[ArrayMinimum(ma,StochasticLength,i)];
         double max = ma[ArrayMaximum(ma,StochasticLength,i)];
               stoch1[i] = (ma[i]-min);
               stoch2[i] = (max  -min);
               double sum1=0,sum2=0; 
                     for (j=0; j<StochasticSlowing; j++)
                     {
                        sum1 += stoch1[i+j];
                        sum2 += stoch2[i+j];
                     }
               double stoch = 0; if (sum2!=0) stoch = 100.0*sum1/sum2;
               double temp  = 0.1 * (stoch - 50.0) ;
            
         //
         //
         //
         //
         //
                  
         ift[i] = ((MathExp(2*temp)-1.0)/(MathExp(2*temp)+1.0)+1.0)*50.0;
         trend[i] = 0;
            if (ift[i]>LevelUp)   trend[i] =  1;
            if (ift[i]<LevelDown) trend[i] = -1;
            manageArrow(i);
      }
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i = limit, r=Bars-i-1; i >= 0; i--,r++)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         ift[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",RwmaDepth,RwmaLength,RwmaPrice,StochasticLength,StochasticSlowing,LevelUp,LevelDown,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,arrowsOn,arrowsIdentifier,arrowsUpColor,arrowsDnColor,arrowsUpGap,arrowsDownGap,0,y);

         //
         //
         //
         //
         //
      
         if (!Interpolate || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;

         //
         //
         //
         //
         //

            datetime time = iTime(NULL,timeFrame,y);
               for(n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
               for(k = 1; k < n; k++)
                  ift[i+k] = ift[i] + (ift[i+n] - ift[i] )*k/n;
   }
   return(0);         
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageArrow(int i)
{
   if (arrowsOn)
   {
      ObjectDelete(arrowsIdentifier+":"+Time[i]);
      if (trend[i]!=trend[i+1])
      {
         if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpGap  ,233,false);
         if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDownGap,234,true);
      }
   }
}               

//
//
//
//
//

void drawArrow(int i,color theColor, double theGap,int theCode,bool up)
{
   string name = arrowsIdentifier+":"+Time[i];
   double gap  = theGap*iATR(NULL,0,20,i);
   
      //
      //
      //
      //
      //
      
      ObjectCreate(name,OBJ_ARROW,0,Time[i],0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[i]+gap);
         else  ObjectSet(name,OBJPROP_PRICE1,Low[i] -gap);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; 
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,DoubleToStr(LevelUp,2)+" crossed up");
         if (trend[whichBar] == -1) doAlert(whichBar,DoubleToStr(LevelDown,2)+" crossed down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message = timeFrameToString(Period())+" "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" value "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," IFT Stochastic of Rainbow WMA "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}